<?php
/**
 * WPEngine Quick Plugin Check
 *
 * @author      Robin Metcalfe
 * @copyright   2017 Solarise Design
 * @license     GPL-2.0+
 *
 * @wordpress-plugin
 * Plugin Name: WPEngine Quick Plugin Check
 * Plugin URI:  https://solarisedesign.co.uk/
 * Description: This plugin scans through all of your installed plugins and flags a warning if any are disallowed by WPEngine. Handy if you're transferring a Wordpress site from another location to WPEngine
 * Version:     1.0.0
 * Author:      Robin Metcalfe
 * Author URI:  https://solarisedesign.co.uk
 * Text Domain: wpengine-quick-plugin-check
 */

class WPEngineQuickPluginCheck
{

	protected $issues = array();
	protected $transientName;

	function __construct()
	{
		add_action('admin_init', array($this, 'init'));
		add_action('admin_notices', array($this, 'adminNoticeHandler'));
	}

	/**
	 * Initialise and scan through plugin list, prepare admin notices
	 */
	public function init()
	{

		$issueList = [];

		$this->transientName = 'admin_custom_notification_wpengine_plugin_check_'.get_current_user_id();

		// Clear out previous notices, incase we are in a page redirect
		delete_transient($this->transientName);

		if(!is_admin()) {
			return;
		}

		if ( ! function_exists( 'get_plugins' ) ) {
			require_once ABSPATH . 'wp-admin/includes/plugin.php';
		}
		$plugins = get_plugins();

		$pluginListFile = dirname(__FILE__) . '/disallowed-list';

		if(is_readable($pluginListFile)) {
			$pluginList = file($pluginListFile, FILE_IGNORE_NEW_LINES);
		}

		foreach($plugins as $key => $data) {

			$parts = pathinfo($key);
			
			if($parts) {
				$dirname = $parts['dirname'];

				if($dirname == ".") {
					$pluginName = $parts['basename'];
				} else {
					$pluginName = $dirname;
				}

				// also check files...
				if(in_array($pluginName, $pluginList)) {
					$issueList[] = "<b>{$data['Name']}</b>";
				}
			}
		}

		if($issueList) {
			$this->addNotice(sprintf("WPEngine compatibility: Please remove the following plugins before uploading: %s", implode(", ", $issueList)), "warning");
		}

	}


	/**
	 * Hooks into WP's admin notification output system
	 * @return [type] [description]
	 */
	public function adminNoticeHandler() {

	    // Check if there are any notices stored
	    $notifications = get_transient($this->transientName);

	    if($notifications):
	        foreach($notifications as $notification):
	            echo <<<HTML

	                <div class="notice notice-custom notice-{$notification['type']} is-dismissible">
	                    <p>{$notification['message']}</p>
	                </div>

HTML;
	        endforeach;
	    endif;

	    // Clear away our transient data, it's not needed any more
	    delete_transient($this->transientName);

	}
	

	/**
	 * Helper function. Adds a notice to the admin notification queue
	 * @param string $message The message to display
	 * @param type $type      "warning", "info", "error", "success"
	 */
	private function addNotice($message, $type)
	{

	    if(!is_admin()) {
	        return false;
	    }

	    // todo: check these are valid
	    if(!in_array($type, array('error', 'info', 'success', 'warning'))) {
	        return false;
	    }

	    // Check if this transient already exists. We can use this to add
	    // multiple notifications during a single pass through our code
	    $notifications = get_transient($this->transientName);

	    if(!$notifications) {
	        $notifications = array(); // initialise as a blank array
	    }

	    $notifications[] = array(
	        'message' => $message,
	        'type' => $type
	    );

	    $res = set_transient($this->transientName, $notifications); 

	}

}

new WPEngineQuickPluginCheck;